@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2024 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@ sabnzbd.interface - webinterface
 
 import os
 import time
-from datetime import datetime
+import datetime
 import cherrypy
 import logging
 import urllib.parse
@@ -47,19 +47,21 @@ from sabnzbd.misc import (
     get_base_url,
     is_ipv4_addr,
     is_ipv6_addr,
-    get_server_addrinfo,
     is_lan_addr,
+    is_local_addr,
     is_loopback_addr,
-    ip_in_subnet,
     helpful_warning,
     recursive_html_escape,
+    is_none,
+    get_cpu_name,
 )
+from sabnzbd.happyeyeballs import happyeyeballs
 from sabnzbd.filesystem import (
     real_path,
     globber,
     globber_full,
     clip_path,
-    same_file,
+    same_directory,
     setname_from_path,
 )
 from sabnzbd.encoding import xml_name, utob
@@ -68,7 +70,6 @@ import sabnzbd.cfg as cfg
 import sabnzbd.notifier as notifier
 import sabnzbd.newsunpack
 from sabnzbd.utils.servertests import test_nntp_server_dict
-from sabnzbd.utils.getperformance import getcpu
 import sabnzbd.utils.ssdp
 from sabnzbd.constants import (
     DEF_STD_CONFIG,
@@ -80,6 +81,7 @@ from sabnzbd.constants import (
     GUESSIT_SORT_TYPES,
     VALID_NZB_FILES,
     VALID_ARCHIVES,
+    DEF_TIMEOUT,
 )
 from sabnzbd.lang import list_languages
 from sabnzbd.api import (
@@ -179,8 +181,7 @@ def secured_expose(
 
         # Some pages need correct API key
         if check_api_key:
-            msg = check_apikey(kwargs)
-            if msg:
+            if msg := check_apikey(kwargs):
                 cherrypy.response.status = 403
                 if cfg.api_warnings():
                     return msg
@@ -206,15 +207,15 @@ def check_access(access_type: int = 4, warn_user: bool = False) -> bool:
 
     remote_ip = cherrypy.request.remote.ip
 
-    # Check for localhost
-    if is_loopback_addr(remote_ip):
-        return True
+    # Check if the client IP is a loopback address or considered local
+    is_allowed = is_loopback_addr(remote_ip) or is_local_addr(remote_ip)
 
-    if not cfg.local_ranges():
-        # No local ranges defined, allow all private addresses by default
-        is_allowed = is_lan_addr(remote_ip)
-    else:
-        is_allowed = any(ip_in_subnet(remote_ip, r) for r in cfg.local_ranges())
+    # Never check the XFF header unless access would have been granted based on the remote IP alone!
+    if is_allowed and cfg.verify_xff_header() and (xff_header := cherrypy.request.headers.get("X-Forwarded-For")):
+        xff_ips = [ip.strip() for ip in xff_header.split(",")]
+        is_allowed = all(is_local_addr(ip) or is_loopback_addr(ip) for ip in xff_ips)
+        if not is_allowed:
+            logging.debug("Denying access based on X-Forwarded-For header '%s'", xff_header)
 
     if not is_allowed and warn_user:
         log_warning_and_ip(T("Refused connection from:"))
@@ -429,7 +430,7 @@ class MainPage:
             info["have_rss_defined"] = bool(config.get_rss())
             info["have_watched_dir"] = bool(cfg.dirscan_dir())
 
-            info["cpumodel"] = getcpu()
+            info["cpumodel"] = get_cpu_name()
             info["cpusimd"] = sabnzbd.decoder.SABCTOOLS_SIMD
 
             # Have logout only with HTML and if inet=5, only when we are external
@@ -469,8 +470,7 @@ class MainPage:
     def scriptlog(self, **kwargs):
         """Needed for all skins, URL is fixed due to postproc"""
         # No session key check, due to fixed URLs
-        name = kwargs.get("name")
-        if name:
+        if name := kwargs.get("name"):
             history_db = sabnzbd.get_db_connection()
             return ShowString(history_db.get_name(name), history_db.get_script_log(name))
         else:
@@ -518,7 +518,6 @@ class Wizard:
             cfg.language.set(kwargs.get("lang"))
 
         info = build_header(sabnzbd.WIZARD_DIR)
-        info["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
         # Just in case, add server
         servers = config.get_servers()
@@ -683,13 +682,8 @@ class ConfigPage:
         conf["configfn"] = clip_path(config.get_filename())
         conf["cmdline"] = sabnzbd.CMDLINE
         conf["build"] = sabnzbd.__baseline__[:7]
-
-        conf["have_unzip"] = bool(sabnzbd.newsunpack.ZIP_COMMAND)
         conf["have_7zip"] = bool(sabnzbd.newsunpack.SEVENZIP_COMMAND)
-        conf["have_sabctools"] = sabnzbd.decoder.SABCTOOLS_ENABLED
-        conf["have_mt_par2"] = sabnzbd.newsunpack.PAR2_MT
-
-        conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
+        conf["have_par2_turbo"] = sabnzbd.newsunpack.PAR2_TURBO
         conf["ssl_version"] = ssl.OPENSSL_VERSION
 
         return template_filtered_response(
@@ -740,21 +734,9 @@ class ConfigFolders:
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveDirectories(self, **kwargs):
         for kw in LIST_DIRPAGE + LIST_BOOL_DIRPAGE:
-            value = kwargs.get(kw)
-            if value is not None or kw in LIST_BOOL_DIRPAGE:
-                if kw in ("complete_dir", "dirscan_dir", "backup_dir"):
-                    msg = config.get_config("misc", kw).set(value, create=True)
-                else:
-                    msg = config.get_config("misc", kw).set(value)
-                if msg:
-                    # return sabnzbd.api.report('json', error=msg)
-                    return badParameterResponse(msg, kwargs.get("ajax"))
+            if msg := config.get_config("misc", kw).set(kwargs.get(kw)):
+                return badParameterResponse(msg, kwargs.get("ajax"))
 
-        if not sabnzbd.filesystem.check_incomplete_vs_complete():
-            return badParameterResponse(
-                T("The Completed Download Folder cannot be the same or a subfolder of the Temporary Download Folder"),
-                kwargs.get("ajax"),
-            )
         config.save_config()
         if kwargs.get("ajax"):
             return sabnzbd.api.report()
@@ -782,11 +764,11 @@ SWITCH_LIST = (
     "nice",
     "ionice",
     "pre_script",
+    "end_queue_script",
     "pause_on_pwrar",
     "sfv_check",
     "deobfuscate_final_filenames",
     "folder_rename",
-    "load_balancing",
     "quota_size",
     "quota_day",
     "quota_resume",
@@ -797,10 +779,9 @@ SWITCH_LIST = (
     "fail_hopeless_jobs",
     "enable_all_par",
     "enable_recursive",
-    "no_series_dupes",
-    "series_propercheck",
+    "no_smart_dupes",
+    "dupes_propercheck",
     "script_can_fail",
-    "new_nzb_on_failure",
     "unwanted_extensions",
     "action_on_unwanted_extensions",
     "unwanted_extensions_mode",
@@ -816,8 +797,6 @@ class ConfigSwitches:
     @secured_expose(check_configlock=True)
     def index(self, **kwargs):
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
-
-        conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
         conf["have_nice"] = bool(sabnzbd.newsunpack.NICE_COMMAND)
         conf["have_ionice"] = bool(sabnzbd.newsunpack.IONICE_COMMAND)
 
@@ -836,8 +815,7 @@ class ConfigSwitches:
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSwitches(self, **kwargs):
         for kw in SWITCH_LIST:
-            msg = config.get_config("misc", kw).set(kwargs.get(kw))
-            if msg:
+            if msg := config.get_config("misc", kw).set(kwargs.get(kw)):
                 return badParameterResponse(msg, kwargs.get("ajax"))
 
         config.save_config()
@@ -852,6 +830,7 @@ SPECIAL_BOOL_LIST = (
     "start_paused",
     "preserve_paused_state",
     "no_penalties",
+    "ipv6_servers",
     "fast_fail",
     "overwrite_files",
     "enable_par_cleanup",
@@ -860,8 +839,8 @@ SPECIAL_BOOL_LIST = (
     "api_warnings",
     "helpful_warnings",
     "ampm",
+    "enable_multipar",
     "enable_unrar",
-    "enable_unzip",
     "enable_7zip",
     "enable_filejoin",
     "enable_tsjoin",
@@ -872,6 +851,7 @@ SPECIAL_BOOL_LIST = (
     "ipv6_hosting",
     "keep_awake",
     "empty_postproc",
+    "new_nzb_on_failure",
     "html_login",
     "wait_for_dfolder",
     "enable_broadcast",
@@ -881,6 +861,7 @@ SPECIAL_BOOL_LIST = (
     "x_frame_options",
     "allow_old_ssl_tls",
     "enable_season_sorting",
+    "verify_xff_header",
 )
 SPECIAL_VALUE_LIST = (
     "downloader_sleep_time",
@@ -894,7 +875,6 @@ SPECIAL_VALUE_LIST = (
     "receive_threads",
     "switchinterval",
     "direct_unpack_threads",
-    "ipv6_servers",
     "selftest_host",
     "ssdp_broadcast_interval",
 )
@@ -935,10 +915,7 @@ class ConfigSpecial:
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSpecial(self, **kwargs):
         for kw in SPECIAL_BOOL_LIST + SPECIAL_VALUE_LIST + SPECIAL_LIST_LIST:
-            item = config.get_config("misc", kw)
-            value = kwargs.get(kw)
-            msg = item.set(value)
-            if msg:
+            if msg := config.get_config("misc", kw).set(kwargs.get(kw)):
                 return badParameterResponse(msg)
 
         config.save_config()
@@ -962,6 +939,8 @@ GENERAL_LIST = (
     "socks5_proxy_url",
     "auto_browser",
     "check_new_rel",
+    "bandwidth_max",
+    "bandwidth_perc",
 )
 
 
@@ -973,8 +952,6 @@ class ConfigGeneral:
     def index(self, **kwargs):
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
 
-        conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
-
         web_list = []
         for interface_dir in globber_full(sabnzbd.DIR_INTERFACES):
             # Ignore the config
@@ -996,8 +973,6 @@ class ConfigGeneral:
         for kw in GENERAL_LIST:
             conf[kw] = config.get_config("misc", kw)()
 
-        conf["bandwidth_max"] = cfg.bandwidth_max()
-        conf["bandwidth_perc"] = cfg.bandwidth_perc()
         conf["nzb_key"] = cfg.nzb_key()
         conf["caller_url"] = cherrypy.request.base + cfg.url_base()
 
@@ -1010,10 +985,7 @@ class ConfigGeneral:
     def saveGeneral(self, **kwargs):
         # Handle general options
         for kw in GENERAL_LIST:
-            item = config.get_config("misc", kw)
-            value = kwargs.get(kw)
-            msg = item.set(value)
-            if msg:
+            if msg := config.get_config("misc", kw).set(kwargs.get(kw)):
                 return badParameterResponse(msg, ajax=kwargs.get("ajax"))
 
         # Handle special options
@@ -1022,16 +994,6 @@ class ConfigGeneral:
         web_dir = kwargs.get("web_dir")
         change_web_dir(web_dir)
 
-        bandwidth_max = kwargs.get("bandwidth_max")
-        if bandwidth_max is not None:
-            cfg.bandwidth_max.set(bandwidth_max)
-        bandwidth_perc = kwargs.get("bandwidth_perc")
-        if bandwidth_perc is not None:
-            cfg.bandwidth_perc.set(bandwidth_perc)
-        bandwidth_perc = cfg.bandwidth_perc()
-        if bandwidth_perc and not bandwidth_max:
-            helpful_warning(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
-
         config.save_config()
 
         # Update CherryPy authentication
@@ -1103,7 +1065,6 @@ class ConfigServer:
 
         conf["servers"] = new
         conf["cats"] = list_cats(default=True)
-        conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
         return template_filtered_response(
             file=os.path.join(sabnzbd.WEB_DIR_CONFIG, "config_server.tmpl"),
@@ -1164,17 +1125,6 @@ def unique_svr_name(server):
     return new_name
 
 
-def check_server(host, port, ajax):
-    """Check if server address resolves properly"""
-    if host.lower() == "localhost" and sabnzbd.AMBI_LOCALHOST:
-        return badParameterResponse(T("Warning: LOCALHOST is ambiguous, use numerical IP-address."), ajax)
-
-    if get_server_addrinfo(host, int_conv(port)):
-        return ""
-    else:
-        return badParameterResponse(T('Server address "%s:%s" is not valid.') % (host, port), ajax)
-
-
 def handle_server(kwargs, root=None, new_svr=False):
     """Internal server handler"""
     ajax = kwargs.get("ajax")
@@ -1194,9 +1144,8 @@ def handle_server(kwargs, root=None, new_svr=False):
         kwargs["connections"] = "1"
 
     if kwargs.get("enable") == "1":
-        msg = check_server(host, port, ajax)
-        if msg:
-            return msg
+        if not happyeyeballs(host, int_conv(port), int_conv(kwargs.get("timeout"), default=DEF_TIMEOUT)):
+            return badParameterResponse(T('Server address "%s:%s" is not valid.') % (host, port), ajax)
 
     # Default server name is just the host name
     server = host
@@ -1429,7 +1378,7 @@ class ConfigRss:
             raise rssRaiser(self.__root, kwargs)
 
         pp = kwargs.get("pp", "")
-        if pp.lower() == "none":
+        if is_none(pp):
             pp = ""
         script = ConvertSpecials(kwargs.get("script"))
         cat = ConvertSpecials(kwargs.get("cat"))
@@ -1529,16 +1478,24 @@ class ConfigRss:
         """Download NZB from provider (Download button)"""
         feed = kwargs.get("feed")
         url = kwargs.get("url")
-        nzbname = kwargs.get("nzbname")
-        att = sabnzbd.RSSReader.lookup_url(feed, url)
-        if att:
+        if att := sabnzbd.RSSReader.lookup_url(feed, url):
+            nzbname = kwargs.get("nzbname")
             pp = att.get("pp")
             cat = att.get("cat")
             script = att.get("script")
-            prio = att.get("prio")
+            priority = att.get("prio")
 
             if url:
-                sabnzbd.urlgrabber.add_url(url, pp, script, cat, prio, nzbname)
+                logging.info("Adding %s (%s) to queue", url, nzbname)
+                sabnzbd.urlgrabber.add_url(
+                    url,
+                    pp=pp,
+                    script=script,
+                    cat=cat,
+                    priority=priority,
+                    nzbname=nzbname,
+                    nzo_info={"RSS": feed},
+                )
             # Need to pass the title instead
             sabnzbd.RSSReader.flag_downloaded(feed, url)
         raise rssRaiser(self.__root, kwargs)
@@ -1780,12 +1737,12 @@ class ConfigCats:
         conf["defdir"] = cfg.complete_dir.get_clipped_path()
 
         categories = config.get_ordered_categories()
-        conf["have_cats"] = len(categories) > 1
+        new_cat_order = max(cat["order"] for cat in categories) + 1
 
-        # Add empty line
+        # Add empty line to add new categories
         empty = {
             "name": "",
-            "order": "0",
+            "order": str(new_cat_order),
             "pp": "-1",
             "script": "",
             "dir": "",
@@ -1816,7 +1773,7 @@ class ConfigCats:
 
         if newname:
             # Check if this cat-dir is not sub-folder of incomplete
-            if same_file(cfg.download_dir.get_path(), real_path(cfg.complete_dir.get_path(), kwargs["dir"])):
+            if same_directory(cfg.download_dir.get_path(), real_path(cfg.complete_dir.get_path(), kwargs["dir"])):
                 return T("Category folder cannot be a subfolder of the Temporary Download Folder.")
 
             # Delete current one and replace with new one
@@ -1979,7 +1936,7 @@ def GetRssLog(feed):
 
         # And we add extra fields for sorting
         if job.get("age", 0):
-            job["age_ms"] = (job["age"] - datetime.utcfromtimestamp(0)).total_seconds()
+            job["age_ms"] = (job["age"] - datetime.datetime(1970, 1, 1)).total_seconds()
             job["age"] = calc_age(job["age"], True)
         else:
             job["age_ms"] = ""
@@ -2162,7 +2119,7 @@ class ConfigNotify:
     @secured_expose(check_configlock=True)
     def index(self, **kwargs):
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
-        conf["notify_types"] = sabnzbd.notifier.NOTIFICATION
+        conf["notify_types"] = sabnzbd.notifier.NOTIFICATION_TYPES
         conf["categories"] = list_cats(False)
         conf["have_ntfosd"] = sabnzbd.notifier.have_ntfosd()
         conf["have_ncenter"] = sabnzbd.MACOS and sabnzbd.FOUNDATION
@@ -2182,7 +2139,8 @@ class ConfigNotify:
     def saveNotify(self, **kwargs):
         for section in NOTIFY_OPTIONS:
             for option in NOTIFY_OPTIONS[section]:
-                config.get_config(section, option).set(kwargs.get(option))
+                if msg := config.get_config(section, option).set(kwargs.get(option)):
+                    return badParameterResponse(msg, kwargs.get("ajax"))
         config.save_config()
         if kwargs.get("ajax"):
             return sabnzbd.api.report()
