@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2024 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -39,7 +39,15 @@ from sabnzbd.newsunpack import (
     is_sfv_file,
 )
 from threading import Thread
-from sabnzbd.misc import on_cleanup_list, is_sample, helpful_warning
+from sabnzbd.misc import (
+    on_cleanup_list,
+    is_sample,
+    helpful_warning,
+    history_updated,
+    change_queue_complete_action,
+    run_script,
+    is_none,
+)
 from sabnzbd.filesystem import (
     real_path,
     get_unique_dir,
@@ -64,6 +72,7 @@ from sabnzbd.filesystem import (
     get_ext,
     get_filename,
     directory_is_writable,
+    check_filesystem_capabilities,
 )
 from sabnzbd.nzbstuff import NzbObject
 from sabnzbd.sorting import Sorter
@@ -145,7 +154,7 @@ class PostProcessor(Thread):
             logging.info("Corrupt %s file, discarding", POSTPROC_QUEUE_FILE_NAME)
             logging.info("Traceback: ", exc_info=True)
 
-    def delete(self, nzo_id, del_files=False):
+    def delete(self, nzo_id: str, del_files: bool = False):
         """Remove a job from the post processor queue"""
         for nzo in self.history_queue:
             if nzo.nzo_id == nzo_id:
@@ -171,7 +180,7 @@ class PostProcessor(Thread):
         else:
             self.slow_queue.put(nzo)
         self.save()
-        sabnzbd.misc.history_updated()
+        history_updated()
 
     def remove(self, nzo: NzbObject):
         """Remove given nzo from the queue"""
@@ -180,7 +189,7 @@ class PostProcessor(Thread):
         except:
             pass
         self.save()
-        sabnzbd.misc.history_updated()
+        history_updated()
 
     def stop(self):
         """Stop thread after finishing running job"""
@@ -188,7 +197,7 @@ class PostProcessor(Thread):
         self.slow_queue.put(None)
         self.fast_queue.put(None)
 
-    def cancel_pp(self, nzo_id):
+    def cancel_pp(self, nzo_id: str) -> Optional[bool]:
         """Change the status, so that the PP is canceled"""
         for nzo in self.history_queue:
             if nzo.nzo_id == nzo_id:
@@ -204,15 +213,45 @@ class PostProcessor(Thread):
                 return True
         return None
 
-    def empty(self):
+    def empty(self) -> bool:
         """Return True if pp queue is empty"""
         return self.slow_queue.empty() and self.fast_queue.empty() and not self.__busy
 
-    def get_queue(self):
-        """Return list of NZOs that still need to be processed"""
-        return [nzo for nzo in self.history_queue if nzo.work_name]
+    def get_queue(
+        self,
+        search: Optional[str] = None,
+        categories: Optional[List[str]] = None,
+        statuses: Optional[List[str]] = None,
+        nzo_ids: Optional[List[str]] = None,
+    ) -> List[NzbObject]:
+        """Return list of NZOs that still need to be processed.
+        Optionally filtered by the search terms"""
+        re_search = None
+        if isinstance(search, str):
+            # Replace * with .* and ' ' with .
+            search_text = search.strip().replace("*", ".*").replace(" ", ".*") + ".*?"
+            try:
+                re_search = re.compile(search_text, re.I)
+            except:
+                logging.error(T("Failed to compile regex for search term: %s"), search_text)
 
-    def get_path(self, nzo_id):
+        # Need a copy to prevent race conditions
+        filtered_queue = []
+        for nzo in self.history_queue[:]:
+            if not nzo.work_name:
+                continue
+            if re_search and not re_search.search(nzo.final_name):
+                continue
+            if categories and nzo.cat not in categories:
+                continue
+            if statuses and nzo.status not in statuses:
+                continue
+            if nzo_ids and nzo.nzo_id not in nzo_ids:
+                continue
+            filtered_queue.append(nzo)
+        return filtered_queue
+
+    def get_path(self, nzo_id: str) -> Optional[str]:
         """Return download path for given nzo_id or None when not found"""
         for nzo in self.history_queue:
             if nzo.nzo_id == nzo_id:
@@ -231,8 +270,10 @@ class PostProcessor(Thread):
         else:
             logging.debug("Completed Download Folder %s is not on FAT", complete_dir)
 
-        directory_is_writable(sabnzbd.cfg.download_dir.get_path())
-        directory_is_writable(sabnzbd.cfg.complete_dir.get_path())
+        if directory_is_writable(sabnzbd.cfg.download_dir.get_path()):
+            check_filesystem_capabilities(sabnzbd.cfg.download_dir.get_path())
+        if directory_is_writable(sabnzbd.cfg.complete_dir.get_path()):
+            check_filesystem_capabilities(sabnzbd.cfg.complete_dir.get_path())
 
         # Do an extra purge of the history on startup to ensure timely removal on systems that
         # aren't on 24/7 and typically don't benefit from the daily scheduled call at midnight
@@ -310,7 +351,7 @@ class PostProcessor(Thread):
             sabnzbd.Downloader.resume_from_postproc()
 
 
-def process_job(nzo: NzbObject):
+def process_job(nzo: NzbObject) -> bool:
     """Process one job"""
     start = time.time()
 
@@ -395,7 +436,7 @@ def process_job(nzo: NzbObject):
                 return False
 
         # If we don't need extra par2, we can disconnect
-        if sabnzbd.NzbQueue.actives(grabs=False) == 0 and cfg.autodisconnect():
+        if not sabnzbd.NzbQueue.actives(grabs=False) and cfg.autodisconnect():
             # This was the last job, close server connections
             sabnzbd.Downloader.disconnect()
 
@@ -481,7 +522,6 @@ def process_job(nzo: NzbObject):
                     # Full cleanup including nzb's
                     cleanup_list(tmp_workdir_complete, skip_nzb=False)
 
-        script_output = ""
         script_ret = 0
         script_error = False
         if not nzb_list:
@@ -502,7 +542,7 @@ def process_job(nzo: NzbObject):
                     )
                     logging.info("Traceback: ", exc_info=True)
                     # Better disable sorting because filenames are all off now
-                    file_sorter.sorter_active = None
+                    file_sorter.sorter_active = False
 
             if empty:
                 job_result = -1
@@ -515,7 +555,7 @@ def process_job(nzo: NzbObject):
             # TV/Movie/Date Renaming code part 2 - rename and move files to parent folder
             if all_ok and file_sorter.sorter_active:
                 if newfiles:
-                    workdir_complete, ok = file_sorter.sorter.rename(newfiles, workdir_complete)
+                    workdir_complete, ok = file_sorter.rename(newfiles, workdir_complete)
                     if not ok:
                         nzo.set_unpack_info("Unpack", T("Failed to move files"))
                         nzo.fail_msg = T("Failed to move files")
@@ -534,8 +574,7 @@ def process_job(nzo: NzbObject):
                     deobfuscate.deobfuscate(nzo, newfiles, nzo.final_name)
 
                 # Run the user script
-                script_path = make_script_path(script)
-                if script_path:
+                if script_path := make_script_path(script):
                     # Set the current nzo status to "Ext Script...". Used in History
                     nzo.status = Status.RUNNING
                     nzo.set_action_line(T("Running script"), script)
@@ -543,19 +582,23 @@ def process_job(nzo: NzbObject):
                     script_log, script_ret = external_processing(
                         script_path, nzo, clip_path(workdir_complete), nzo.final_name, job_result
                     )
+
+                    # Format output depending on return status
                     script_line = get_last_line(script_log)
-                    if script_log:
-                        script_output = nzo.nzo_id
-                    if script_line:
-                        nzo.set_unpack_info("Script", script_line, unique=True)
-                    else:
-                        nzo.set_unpack_info("Script", T("Ran %s") % script, unique=True)
+                    if script_ret:
+                        if script_line:
+                            script_line = "Exit(%s): %s " % (script_ret, script_line)
+                        else:
+                            script_line = T("Script exit code is %s") % script_ret
+                    elif not script_line:
+                        script_line = T("Ran %s") % script
+                    nzo.set_unpack_info("Script", script_line, unique=True)
 
                     # Maybe bad script result should fail job
                     if script_ret and cfg.script_can_fail():
                         script_error = True
                         all_ok = False
-                        nzo.fail_msg = T("Script exit code is %s") % script_ret
+                        nzo.fail_msg = script_line
 
         # Email the results
         if not nzb_list and cfg.email_endjob():
@@ -573,29 +616,24 @@ def process_job(nzo: NzbObject):
                     script_ret,
                 )
 
-        if script_output:
+        if script_log and len(script_log.rstrip().split("\n")) > 1:
             # Can do this only now, otherwise it would show up in the email
-            if script_ret:
-                script_ret = "Exit(%s) " % script_ret
-            else:
-                script_ret = ""
-            if len(script_log.rstrip().split("\n")) > 1:
-                nzo.set_unpack_info(
-                    "Script",
-                    '%s%s <a href="./scriptlog?name=%s">(%s)</a>' % (script_ret, script_line, script_output, T("More")),
-                    unique=True,
-                )
-            else:
-                # No '(more)' button needed
-                nzo.set_unpack_info("Script", "%s%s " % (script_ret, script_line), unique=True)
+            nzo.set_unpack_info(
+                "Script",
+                '%s <a href="./scriptlog?name=%s">(%s)</a>' % (script_line, nzo.nzo_id, T("More")),
+                unique=True,
+            )
 
         # Cleanup again, including NZB files
-        if all_ok:
+        if all_ok and os.path.isdir(workdir_complete):
             cleanup_list(workdir_complete, False)
 
         # Force error for empty result
         all_ok = all_ok and not empty
 
+        # See if we need to start an alternative or remove the duplicates
+        sabnzbd.NzbQueue.handle_duplicate_alternatives(nzo, all_ok)
+
     except:
         logging.error(T("Post Processing Failed for %s (%s)"), filename, T("see logfile"))
         logging.info("Traceback: ", exc_info=True)
@@ -620,6 +658,7 @@ def process_job(nzo: NzbObject):
                 0,
             )
 
+    workdir_notifcation_action = workdir_complete
     if all_ok:
         # If the folder only contains one file OR folder, have that as the path
         # Be aware that series/generic/date sorting may move a single file into a folder containing other files
@@ -644,7 +683,13 @@ def process_job(nzo: NzbObject):
 
     # Show final status in history
     if all_ok:
-        notifier.send_notification(T("Download Completed"), filename, "complete", nzo.cat)
+        notifier.send_notification(
+            T("Download Completed"),
+            filename,
+            "complete",
+            nzo.cat,
+            {"open_folder": clip_path(workdir_notifcation_action)},
+        )
         nzo.status = Status.COMPLETED
         nzo.fail_msg = ""
     else:
@@ -661,7 +706,7 @@ def process_job(nzo: NzbObject):
         # Purge items
         history_db.auto_history_purge()
 
-    sabnzbd.misc.history_updated()
+    history_updated()
     return True
 
 
@@ -670,32 +715,37 @@ def prepare_extraction_path(nzo: NzbObject) -> Tuple[str, str, Sorter, bool, Opt
     the extraction path and create the directory.
     Separated so it can be called from DirectUnpacker
     """
-    one_folder = False
+    create_job_dir = True
     marker_file = None
 
     # Determine category directory
     catdir = config.get_category(nzo.cat).dir()
     if not catdir:
-        # If none defined, check Default category directory
+        # Fall back to Default if undefined at category-level
         catdir = config.get_category().dir()
 
-    # Check if it should have a directory
+    # Check whether the creation of job directories has been disabled
     if catdir.endswith("*"):
-        catdir = catdir.strip("*")
-        one_folder = True
+        catdir = catdir[:-1]
+        create_job_dir = False
 
-    complete_dir = real_path(cfg.complete_dir.get_path(), catdir)
-    complete_dir = long_path(complete_dir)
+    complete_dir = long_path(real_path(cfg.complete_dir.get_path(), catdir))
 
-    # TV/Movie/Date Renaming code part 1 - detect and construct paths
-    file_sorter = Sorter(nzo, nzo.cat)
-    complete_dir = file_sorter.detect(nzo.final_name, complete_dir)
+    # Initialize the sorter and let it construct a path for the Complete directory
+    file_sorter = Sorter(
+        nzo,
+        nzo.final_name,
+        complete_dir,
+        nzo.cat,
+    )
     if file_sorter.sorter_active:
-        one_folder = False
+        complete_dir = file_sorter.get_final_path()
+        # Sorting overrides the per-category job directory creation setting
+        create_job_dir = True
 
     complete_dir = sanitize_and_trim_path(complete_dir)
 
-    if one_folder:
+    if not create_job_dir:
         workdir_complete = create_all_dirs(complete_dir, apply_permissions=True)
     else:
         workdir_complete = get_unique_dir(os.path.join(complete_dir, nzo.final_name), create_dir=True)
@@ -705,7 +755,7 @@ def prepare_extraction_path(nzo: NzbObject) -> Tuple[str, str, Sorter, bool, Opt
         logging.error(T("Cannot create final folder %s") % os.path.join(complete_dir, nzo.final_name))
         raise IOError
 
-    if cfg.folder_rename() and not one_folder:
+    if create_job_dir and cfg.folder_rename():
         prefixed_path = prefix(workdir_complete, "_UNPACK_")
         tmp_workdir_complete = get_unique_dir(prefix(workdir_complete, "_UNPACK_"), create_dir=False)
 
@@ -716,14 +766,15 @@ def prepare_extraction_path(nzo: NzbObject) -> Tuple[str, str, Sorter, bool, Opt
 
         # Is the unique path different? Then we also need to modify the final path
         if prefixed_path != tmp_workdir_complete:
-            workdir_complete = workdir_complete + os.path.splitext(tmp_workdir_complete)[1]
+            # The unique path adds an "extension"
+            workdir_complete = workdir_complete + get_ext(tmp_workdir_complete)
     else:
         tmp_workdir_complete = workdir_complete
 
-    return tmp_workdir_complete, workdir_complete, file_sorter, one_folder, marker_file
+    return tmp_workdir_complete, workdir_complete, file_sorter, not create_job_dir, marker_file
 
 
-def parring(nzo: NzbObject):
+def parring(nzo: NzbObject) -> Tuple[bool, bool]:
     """Perform par processing. Returns: (par_error, re_add)"""
     logging.info("Starting verification and repair of %s", nzo.final_name)
     par_error = False
@@ -764,7 +815,7 @@ def parring(nzo: NzbObject):
         # This can happen even if par2 is present, it is always performed
         # so that in the next section the try_rar_check can be used if no
         # par2 check was performed in the previous part
-        _, _, rars, _, _ = build_filelists(nzo.download_path, check_rar=False)
+        _, rars, _, _ = build_filelists(nzo.download_path, check_rar=False)
         if not rars:
             # Returns number of renamed RAR's
             rar_renamer(nzo)
@@ -784,7 +835,7 @@ def parring(nzo: NzbObject):
             # If no luck with SFV, do RAR-check
             if sfv_check_result is None and cfg.enable_unrar():
                 # Check for RAR's with a sensible extension
-                _, _, rars, _, _ = build_filelists(nzo.download_path, check_rar=False)
+                _, rars, _, _ = build_filelists(nzo.download_path, check_rar=False)
                 if rars:
                     par_error = not try_rar_check(nzo, rars)
 
@@ -842,7 +893,7 @@ def try_sfv_check(nzo: NzbObject) -> Optional[bool]:
     return True
 
 
-def try_rar_check(nzo: NzbObject, rars):
+def try_rar_check(nzo: NzbObject, rars: List[str]) -> bool:
     """Attempt to verify set using the RARs
     Return True if verified, False when failed
     When setname is '', all RAR files will be used, otherwise only the matching one
@@ -911,7 +962,7 @@ def rar_renamer(nzo: NzbObject) -> int:
         file_to_check = os.path.join(nzo.download_path, file_to_check)
 
         # We only want files:
-        if not (os.path.isfile(file_to_check)):
+        if not os.path.isfile(file_to_check):
             continue
 
         if rarfile.is_rarfile(file_to_check):
@@ -1038,20 +1089,25 @@ def rar_renamer(nzo: NzbObject) -> int:
 
 def handle_empty_queue():
     """Check if empty queue calls for action"""
-    if sabnzbd.NzbQueue.actives() == 0:
+    if not sabnzbd.NzbQueue.actives():
         sabnzbd.save_state()
-        notifier.send_notification("SABnzbd", T("Queue finished"), "queue_done")
+        notifier.send_notification(
+            "SABnzbd",
+            T("Queue finished"),
+            "queue_done",
+            actions={"open_complete": cfg.complete_dir.get_clipped_path()},
+        )
+
+        # Perform end-of-queue script
+        if not is_none(cfg.end_queue_script()):
+            logging.info("Queue has finished, launching script: %s ", cfg.end_queue_script())
+            run_script(cfg.end_queue_script())
 
         # Perform end-of-queue action when one is set
         if sabnzbd.QUEUECOMPLETEACTION:
-            logging.info(
-                "Queue has finished, launching: %s (%s)", sabnzbd.QUEUECOMPLETEACTION, sabnzbd.QUEUECOMPLETEARG
-            )
-            if sabnzbd.QUEUECOMPLETEARG:
-                sabnzbd.QUEUECOMPLETEACTION(sabnzbd.QUEUECOMPLETEARG)
-            else:
-                Thread(target=sabnzbd.QUEUECOMPLETEACTION).start()
-            sabnzbd.misc.change_queue_complete_action(cfg.queue_complete(), new=False)
+            logging.info("Queue has finished, launching action: %s ", sabnzbd.QUEUECOMPLETEACTION)
+            Thread(target=sabnzbd.QUEUECOMPLETEACTION).start()
+            change_queue_complete_action(cfg.queue_complete(), new=False)
 
         # Trigger garbage collection and release of memory
         logging.debug("Triggering garbage collection and release of memory")
@@ -1060,33 +1116,30 @@ def handle_empty_queue():
             sabnzbd.LIBC.malloc_trim(0)
 
 
-def cleanup_list(wdir, skip_nzb):
+def cleanup_list(wdir: str, skip_nzb: bool):
     """Remove all files whose extension matches the cleanup list,
     optionally ignoring the nzb extension
     """
     if cfg.cleanup_list():
         try:
-            files = os.listdir(wdir)
+            with os.scandir(wdir) as files:
+                for entry in files:
+                    if entry.is_dir():
+                        cleanup_list(entry.path, skip_nzb)
+                        cleanup_empty_directories(entry.path)
+                    else:
+                        if on_cleanup_list(entry.name, skip_nzb):
+                            try:
+                                logging.info("Removing unwanted file %s", entry.path)
+                                remove_file(entry.path)
+                            except:
+                                logging.error(T("Removing %s failed"), clip_path(entry.path))
+                                logging.info("Traceback: ", exc_info=True)
         except:
-            files = ()
-        for filename in files:
-            path = os.path.join(wdir, filename)
-            if os.path.isdir(path):
-                cleanup_list(path, skip_nzb)
-            else:
-                if on_cleanup_list(filename, skip_nzb):
-                    try:
-                        logging.info("Removing unwanted file %s", path)
-                        remove_file(path)
-                    except:
-                        logging.error(T("Removing %s failed"), clip_path(path))
-                        logging.info("Traceback: ", exc_info=True)
-        if files:
-            # If directories only contained unwanted files, remove them
-            cleanup_empty_directories(wdir)
+            logging.info("Traceback: ", exc_info=True)
 
 
-def prefix(path, pre):
+def prefix(path: str, pre: str) -> str:
     """Apply prefix to last part of path
     '/my/path' and 'hi_' will give '/my/hi_path'
     """
@@ -1124,7 +1177,7 @@ def nzb_redirect(wdir, nzbname, pp, script, cat, priority):
     return files
 
 
-def one_file_or_folder(folder):
+def one_file_or_folder(folder: str) -> str:
     """If the dir only contains one file or folder, join that file/folder onto the path"""
     if os.path.exists(folder) and os.path.isdir(folder):
         try:
@@ -1141,7 +1194,7 @@ def one_file_or_folder(folder):
 TAG_RE = re.compile(r"<[^>]+>")
 
 
-def get_last_line(txt):
+def get_last_line(txt: str) -> str:
     """Return last non-empty line of a text, trim to 150 max"""
     # First we remove HTML code in a basic way
     txt = TAG_RE.sub(" ", txt)
@@ -1158,7 +1211,7 @@ def get_last_line(txt):
     return line
 
 
-def remove_samples(path):
+def remove_samples(path: str):
     """Remove all files that match the sample pattern
     Skip deleting if it matches all files or there is only 1 file
     """
@@ -1183,7 +1236,7 @@ def remove_samples(path):
         logging.info("Skipping sample-removal, false-positive")
 
 
-def rename_and_collapse_folder(oldpath, newpath, files):
+def rename_and_collapse_folder(oldpath: str, newpath: str, files: List[str]) -> List[str]:
     """Rename folder, collapsing when there's just a single subfolder
     oldpath --> newpath OR oldpath/subfolder --> newpath
     Modify list of filenames accordingly
@@ -1211,8 +1264,7 @@ def rename_and_collapse_folder(oldpath, newpath, files):
 
 def set_marker(folder: str) -> Optional[str]:
     """Set marker file and return name"""
-    name = cfg.marker_file()
-    if name:
+    if name := cfg.marker_file():
         path = os.path.join(folder, name)
         logging.debug("Create marker file %s", path)
         try:
@@ -1236,7 +1288,7 @@ def del_marker(path: str):
             logging.info("Traceback: ", exc_info=True)
 
 
-def remove_from_list(name, lst):
+def remove_from_list(name: Optional[str], lst: List[str]):
     if name:
         for n in range(len(lst)):
             if lst[n].endswith(name):
@@ -1245,7 +1297,7 @@ def remove_from_list(name, lst):
                 return
 
 
-def try_alt_nzb(nzo):
+def try_alt_nzb(nzo: NzbObject):
     """Try to get a new NZB if available"""
     url = nzo.nzo_info.get("failure")
     if url and cfg.new_nzb_on_failure():
