@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -OO
 # -*- coding: UTF-8 -*-
-# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2024 by The SABnzbd-Team (sabnzbd.org)
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@ SKIN_TEXT = {
     "stage-unpack": TT("Unpack"),  #: PP phase "unpack"
     "stage-deobfuscate": TT("Deobfuscate"),  #: PP phase "deobfuscate"
     "stage-script": TT("Script"),  #: PP phase "script"
+    "stage-rss": TT("RSS"),  #: PP RSS feed  of the NZB
     "stage-source": TT("Source"),  #: PP Source of the NZB (path or URL)
     "stage-servers": TT("Servers"),  #: PP Distribution over servers
     "post-Completed": TT("Completed"),  #: PP status
@@ -105,7 +106,7 @@ SKIN_TEXT = {
     "notAvailable": TT("Not available"),
     "homePage": TT("Home page"),  #: Home page of the SABnzbd project
     "source": TT("Source"),  #: Where to find the SABnzbd sourcecode
-    "or": TT("or"),  #: Used in "IRC or IRC-Webaccess"
+    "or": TT("or"),
     "host": TT("Host"),
     "cancel": TT("Cancel"),
     "login": TT("Log in"),
@@ -125,7 +126,7 @@ SKIN_TEXT = {
     "menu-help": TT("Help"),  #: Main menu item
     "menu-wiki": TT("Wiki"),  #: Main menu item
     "menu-forums": TT("Forum"),  #: Main menu item
-    "menu-irc": TT("IRC"),  #: Main menu item
+    "menu-live-chat": TT("Live Chat"),  #: Main menu item
     "menu-issues": TT("Issues"),  #: Main menu item
     "menu-donate": TT("Support the project, Donate!"),  #: Main menu item
     "cmenu-general": TT("General"),  #: Main menu item
@@ -169,7 +170,6 @@ SKIN_TEXT = {
     "mode": TT("Processing"),  #: Queue page table column header
     "name": TT("Name"),  #: Queue page table column header
     "button-retry": TT("Retry"),  #: Queue page button
-    "eoq-actions": TT("Actions"),  #: Queue end-of-queue selection box
     "eoq-scripts": TT("Scripts"),  #: Queue page table, script selection menu
     "purgeQueue": TT("Purge Queue"),  #: Queue page button
     "purgeQueueConf": TT("Delete all items from the queue?"),  #: Confirmation popup
@@ -251,13 +251,8 @@ SKIN_TEXT = {
     ),
     "confirmWithoutSavingPrompt": TT("Changes have not been saved, and will be lost."),
     "explain-sessionExpire": TT("When your IP address changes or SABnzbd is restarted the session will expire."),
-    "opt-enable_unzip": TT("Enable Unzip"),
     "opt-enable_7zip": TT("Enable 7zip"),
-    "opt-multicore-par2": TT("Multicore Par2"),
-    "explain-nosslcontext": TT(
-        "Secure (SSL) connections from SABnzbd to newsservers and HTTPS websites will be encrypted, however, validating a server's identity using its certificates is not possible. OpenSSL 1.0.2 or above and up-to-date local CA certificates are required."
-    ),
-    "explain-getpar2mt": TT("Speed up repairs by installing multicore Par2, it is available for many platforms."),
+    "explain-getpar2turbo": TT("Speed up repairs by installing par2cmdline-turbo, it is available for many platforms."),
     "version": TT("Version"),
     "uptime": TT("Uptime"),
     "backup": TT("Backup"),  #: Indicates that server is Backup server in Status page
@@ -270,8 +265,8 @@ SKIN_TEXT = {
     "explain-host": TT("Host SABnzbd should listen on."),
     "opt-port": TT("SABnzbd Port"),
     "explain-port": TT("Port SABnzbd should listen on."),
-    "opt-web_dir": TT("Web Interface"),
-    "explain-web_dir": TT("Choose a skin."),
+    "opt-web_dir": TT("Web Interface Theme"),
+    "explain-web_dir": TT("Choose a theme."),
     "opt-web_username": TT("SABnzbd Username"),
     "explain-web_username": TT("Optional authentication username."),
     "opt-web_password": TT("SABnzbd Password"),
@@ -398,6 +393,7 @@ SKIN_TEXT = {
     "explain-admin_dir2": TT("<i>Data will <b>not</b> be moved. Requires SABnzbd restart!</i>"),
     "opt-log_dir": TT("Log Folder"),
     "explain-log_dir": TT("Location of log files for SABnzbd.<br /><i>Requires SABnzbd restart!</i>"),
+    "purge_log_files": TT("Purge Logs"),
     "opt-nzb_backup_dir": TT(".nzb Backup Folder"),
     "explain-nzb_backup_dir": TT("Location where .nzb files will be stored."),
     "base-folder": TT("Default Base Folder"),
@@ -416,17 +412,13 @@ SKIN_TEXT = {
     ),
     "opt-pause_on_pwrar": TT("Action when encrypted RAR is downloaded"),
     "explain-pause_on_pwrar": TT('In case of "Pause", you\'ll need to set a password and resume the job.'),
-    "opt-no_dupes": TT("Detect Duplicate Downloads"),
-    "explain-no_dupes": TT(
-        "Detect identical NZB files (based on items in your History or files in .nzb Backup Folder)"
-    ),
-    "opt-no_series_dupes": TT("Detect duplicate episodes in series"),
-    "explain-no_series_dupes": TT(
-        'Detect identical episodes in series (based on "name/season/episode" of items in your History)'
-    ),
-    "opt-series_propercheck": TT("Allow proper releases"),
-    "explain-series_propercheck": TT(
-        "Bypass series duplicate detection if PROPER, REAL or REPACK is detected in the download name"
+    "opt-no_dupes": TT("Identical download detection"),
+    "explain-no_dupes": TT("Detect identical downloads based on name or NZB contents."),
+    "opt-no_smart_dupes": TT("Smart duplicate detection"),
+    "explain-no_smart_dupes": TT("Detect duplicates based on analysis of the filename."),
+    "opt-dupes_propercheck": TT("Allow proper releases"),
+    "explain-dupes_propercheck": TT(
+        "Bypass smart duplicate detection if PROPER, REAL or REPACK is detected in the download name."
     ),
     "nodupes-off": TT("Off"),  #: Three way switch for duplicates
     "nodupes-ignore": TT("Discard"),  #: Four way switch for duplicates
@@ -448,22 +440,18 @@ SKIN_TEXT = {
     "explain-script_can_fail": TT(
         "When the user script returns a non-zero exit code, the job will be flagged as failed."
     ),
-    "opt-new_nzb_on_failure": TT("On failure, try alternative NZB"),
-    "explain-new_nzb_on_failure": TT("Some servers provide an alternative NZB when a download fails."),
     "opt-folder_rename": TT("Enable folder rename"),
     "explain-folder_rename": TT(
         "Use temporary names during post processing. Disable when your system doesn't handle that properly."
     ),
     "opt-pre_script": TT("Pre-queue user script"),
     "explain-pre_script": TT("Used before an NZB enters the queue."),
+    "opt-end_queue_script": TT("On queue finish script"),
+    "explain-end_queue_script": TT("Executed after the queue finishes downloading."),
     "opt-par_option": TT("Extra PAR2 Parameters"),
-    "explain-par_option": TT("Read the Wiki Help on this!"),
     "opt-nice": TT("Nice Parameters"),
-    "explain-nice": TT("Read the Wiki Help on this!"),
     "opt-ionice": TT("IONice Parameters"),
-    "explain-ionice": TT("Read the Wiki Help on this!"),
     "opt-win_process_prio": TT("External process priority"),
-    "explain-win_process_prio": TT("Read the Wiki Help on this!"),
     "win_process_prio-high": TT("High"),
     "win_process_prio-normal": TT("Normal"),
     "win_process_prio-low": TT("Low"),
@@ -482,7 +470,6 @@ SKIN_TEXT = {
         "Posts will be paused untill they are at least this age. Setting job priority to Force will skip the delay."
     ),
     "opt-check_new_rel": TT("Check for New Release"),
-    "explain-check_new_rel": TT("Weekly check for new SABnzbd release."),
     "also-test": TT("Also test releases"),  #: Pick list for weekly test for new releases
     "opt-replace_spaces": TT("Replace Spaces in Foldername"),
     "explain-replace_spaces": TT("Replace spaces with underscores in folder names."),
@@ -539,11 +526,6 @@ SKIN_TEXT = {
     "explain-fail_hopeless_jobs": TT(
         "When during download it becomes clear that too much data is missing, abort the job"
     ),
-    "opt-load_balancing": TT("Server IP address selection"),
-    "no-load-balancing": TT("First IP address"),
-    "load-balancing": TT("Randomly selected IP address"),
-    "load-balancing-happy-eyeballs": TT("Quickest IP address, preferring IPv6"),
-    "explain-load_balancing": TT("Useful if a newsserver has more than one IPv4/IPv6 address"),
     # Config->Server
     "addServer": TT("Add Server"),  #: Caption
     "srv-displayname": TT("Server description"),  #: User defined name for server
@@ -697,7 +679,6 @@ SKIN_TEXT = {
     "opt-nscript_parameters": TT("Parameters"),  #: Notification Script settings
     "explain-nscript_enable": TT("Executes a custom script"),  #: Notification Scriptsettings
     "explain-nscript_script": TT("Which script should we execute for notification?"),  #: Notification Scriptsettings
-    "explain-nscript_parameters": TT("Read the Wiki Help on this!"),  #: Notification Script settings
     # Config->Cat
     "explain-catTags": TT(
         'Indexers can supply a category inside the NZB which SABnzbd will try to match to the categories defined below. Additionally, you can add terms to "Indexer Categories / Groups" to match more categories. Use commas to separate terms. Wildcards in the terms are supported. <br>More information can be found on the Wiki.'
@@ -850,6 +831,9 @@ SKIN_TEXT = {
     "Glitter-top": TT("Top"),
     "Glitter-bottom": TT("Bottom"),
     "Glitter-retryJob": TT("Retry"),
+    "Glitter-retryNoChecks": TT(
+        "When you Retry a job, 'Duplicate Detection' and 'Abort jobs that cannot be completed' are disabled."
+    ),
     "Glitter-more": TT("More"),
     "Glitter-scriptLog": TT("View Script Log"),
     "Glitter-clearHistory": TT("Purge History"),
